function [dp, message] = open_problem(filename)
%OPEN_PROBLEM Open decision problem.
%   dp = OPEN_PROBLEM(filename) reads the decision problem in the
%   specified file and returns it.  If the open is not successful, an
%   empty matrix is returned.
%
%   [dp,message] = OPEN_PROBLEM(filename) returns a system dependent
%   error message if the open is not successful.
  
  [fid, message] = fopen(filename, 'rt');
  if fid == -1
    dp = [];
  else
    dp.A = read_alternatives(fid);
    [dp.AP, dp.BP, dp.hp, dp.AU, dp.BU, dp.hu] = read_constraints(fid, dp.A);
    fclose(fid);
  end
  
  
function A = read_alternatives(fid)
%READ_ALTERNATIVES Read alternatives.
%   A = READ_ALTERNATIVES(FID) reads a set of alternatives from the
%   given file.
  
  n = 0; % number of alternatives read
  while 1
    pos = ftell(fid);
    if ~strcmp(fgetl(fid), 'A')
      fseek(fid, pos, 'bof');
      if n == 0
	A = [];
      end
      break;
    else
      n = n+1;
      A(n).C = read_consequences(fid);
    end
  end
  
  
function C = read_consequences(fid)
%READ_CONSEQUENCES Read consequences.
%   C = READ_CONSEQUENCES(fid) reads a set of consequences from the
%   given file.
  
  n = 0; % number of consequences read
  while 1
    [pmin, count] = fscanf(fid, '%g', 1);
    if count ~= 1
      if n == 0
	C = [];
      end
      break;
    else
      n = n+1;
      C(n).pmin = pmin;
      C(n).pmax = fscanf(fid, '%g', 1);
      C(n).pdist = fgetl(fid);
      C(n).umin = fscanf(fid, '%g', 1);
      C(n).umax = fscanf(fid, '%g', 1);
      C(n).udist = fgetl(fid);
    end
  end
  
  
function [AP, BP, hp, AU, BU, hu] = read_constraints(fid, A)
%READ_CONSTRAINTS Read constraints.
  
  pos = ftell(fid);
  if ~strcmp(fgetl(fid), 'C')
    fseek(fid, pos, 'bof');
  end
  nc = 0; % total number of consequences
  for k = 1:length(A)
    nc = nc+length(A(k).C);
  end
  AP = [];
  BP = [];
  hp = 0; % number of hard probability constraints
  AU = [];
  BU = [];
  hu = 0; % number of hard utility constraints
  nn = 0;
  for ii = 1:length(A)
    % Create completeness constraint for each consequence set.
    k = length(A(ii).C);
    a = [zeros(1, nn) ones(1, k) zeros(1, nc-nn-k)];
    AP = [a; -a; AP];
    BP = [1; -1; BP];
    hp = hp+2;
    for jj = 1:k
      % Create hard and soft domain constraint for each probability
      % and utility variable.
      a = [zeros(1, nn+jj-1) 1 zeros(1, nc-nn-jj)];
      AP = [-a; a; AP; -a; a];
      BP = [0; 1; BP; -A(ii).C(jj).pmin; A(ii).C(jj).pmax];
      hp = hp+2;
      AU = [-a; a; AU; -a; a];
      BU = [0; 1; BU; -A(ii).C(jj).umin; A(ii).C(jj).umax];
      hu = hu+2;
    end
    nn = nn+k;
  end
  % Read user specified linear constraints.
  EPSILON = 0;
  SIGN = 1;
  REAL = 2;
  VARIABLE = 4;
  RELATION = 8;
  TERM = bitor(REAL, VARIABLE);
  while 1
    a = zeros(1, nc);
    b = 0;
    sign = 1;
    side = 1;
    k = 1;
    prev = EPSILON;
    next = bitor(SIGN, TERM);
    s = fgetl(fid);
    if s == -1
      break;
    end
    n = 1;
    while n <= length(s)
      ch = s(n);
      n = n+1;
      if (ch == '+' | ch == '-') & bitand(next, SIGN)
	if bitand(prev, REAL)
	  b = b-sign*k;
	  k = 1;
	end
	if ch == '+'
	  sign = side;
	else
	  sign = -side;
	end
	prev = SIGN;
	next = TERM;
      elseif (ch == '.' | isdigit(ch)) & bitand(next, REAL)
	n = n-1;
	[k, count, err, nn] = sscanf(s(n:length(s)), '%g', 1);
	n = n+nn-1;
	prev = REAL;
	if side > 0
	  next = bitor(bitor(VARIABLE, SIGN), RELATION);
	else
	  next = bitor(VARIABLE, SIGN);
	end
      elseif (ch == 'p' | ch == 'u') & bitand(next, VARIABLE)
	type = ch;
	ii = 0;
	jj = 0;
	ch = s(n);
	n = n+1;
	if isdigit(ch)
	  ii = str2num(ch);
	  ch = s(n);
	  n = n+1;
	  if isdigit(ch)
	    jj = str2num(ch);
	    if n <= length(s)
	      ch = s(n);
	      n = n+1;
	      if isdigit(ch)
		n = n-3;
		[ii, count, err, nn] = sscanf(s(n:length(s)), '%d', 1);
		n = n+nn-1;
		jj = 0;
		ch = s(n);
		n = n+1;
	      elseif ch == ','
		ii = 10*ii+jj;
		jj = 0;
	      else
		n = n - 1;
	      end
	    end
	  end
	  if jj == 0 & ch == ','
	    [jj, count, err, nn] = sscanf(s(n:length(s)), '%d', 1);
	  end
	end
	nn = 0;
	for m = 1:ii-1
	  nn = nn+length(A(m).C);
	end
	a(nn+jj) = sign*k;
	k = 1;
	prev = VARIABLE;
	if side > 0
	  next = bitor(SIGN, RELATION);
	else
	  next = SIGN;
	end
      elseif (ch == '<' | ch == '>' | ch == '=') & bitand(next, ...
							  RELATION)
	if bitand(prev, REAL)
	  b = b-sign*k;
	  k = 1;
	end
	if ch == '<' | ch == '>'
	  if ch == '<'
	    r = 1;
	  else
	    r = -1;
	  end
	  ch = s(n);
	  n = n+1;
	else
	  r = 0;
	end
	sign = -1;
	side = -1;
	prev = RELATION;
	next = bitor(SIGN, TERM);
      end
    end
    if bitand(prev, REAL)
      b = b-sign*k;
    end
    if r == 0
      if type == 'p'
	AP = [AP; a; -a];
	BP = [BP; b; -b];
      else
	AU = [AU; a; -a];
	BU = [BU; b; -b];
      end
    else
      if type == 'p'
	AP = [AP; r*a];
	BP = [BP; r*b];
      else
	AU = [AU; r*a];
	BU = [BU; r*b];
      end
    end
  end
  
  
function b = isdigit(s)
%ISDIGIT True for digits.
%   For a string s, ISDIGIT(s) is 1 for digits and decimal points and
%   0 otherwise.
  
  b = ~(isletter(s) | isspace(s) | s == '-' | s == '+' | s == '<' | ...
	s == '>' | s == '=');