function [alt, EU, IP, IU, P, U, XIP, XIU] = solve_problem(dp, C)
%SOLVE_PROBLEM Solve decision problem.
%   alt = SOLVE_PROBLEM(dp) solves the given decision problem and
%   returns the alternative with the highest utility.
%
%   alt = SOLVE_PROBLEM(dp,C) allows constraints to be relaxed when
%   solving the problem.  C is a parameter, a larger value
%   corresponding to assigning higher penalty to constraint
%   relaxation.
%
%   [alt,EU] = SOLVE_PROBLEM(dp,C) returns a vector of expected
%   utilities for the alternatives.
%
%   [alt,EU,IP] = SOLVE_PROBLEM(dp,C) also returns the inconsistency
%   measure for the probability base.
%
%   [alt,EU,IP,IU] = SOLVE_PROBLEM(dp,C) also returns the
%   inconsistency measure for the utility base.
%
%   [alt,EU,IP,IU,P] = SOLVE_PROBLEM(dp,C) also returns the
%   probability vector minimizing the inconsistency measure.
%
%   [alt,EU,IP,IU,P,U] = SOLVE_PROBLEM(dp,C) also returns the
%   utility vector minimizing the inconsistency measure.
%
%   [alt,EU,IP,IU,P,U,XIP] = SOLVE_PROBLEM(dp,C) also returns the
%   solution vector for the slack variables used to relax probability
%   constraints.
%
%   [alt,EU,IP,IU,P,U,XIP,XIU] = SOLVE_PROBLEM(dp,C) also returns the
%   solution vector for the slack variables used to relax utility
%   constraints.
  
  nvp = size(dp.AP, 2); % number of probability variables
  ncp = size(dp.AP, 1); % number of probability constraints
  nsp = ncp-dp.hp; % number of soft probability constraints
  nvu = size(dp.AU, 2); % number of utility variables
  ncu = size(dp.AU, 1); % number of utility constraints
  nsu = ncu-dp.hu; % number of soft utility constraints
  PC = pcentroid(dp);
  UC = ucentroid(dp);
  if nargin < 2
    C = 0;
  end
  if C <= 0
    % do not allow constraints to be relaxed
    P = conls(eye(nvp), PC, dp.AP, dp.BP);
    XIP = [];
    IP = 0.5*norm(P-PC)^2;
    U = conls(eye(nvu), UC, dp.AU, dp.BU);
    XIU = [];
    IU = 0.5*norm(U-UC)^2;
  else
    H = diag([ones(nvp, 1); zeros(nsp, 1)]);
    f = [zeros(nvp, 1); C*ones(nsp, 1)];
    A = [dp.AP [zeros(dp.hp, nsp); -eye(nsp)]; zeros(nsp, nvp) -eye(nsp)];
    b = [dp.BP-dp.AP*PC; zeros(nsp, 1)];
    X = qp(H, f, A, b);
    P = X(1:nvp)+PC;
    XIP = X(nvp+1:size(A,2));
    IP = 0.5*X'*H*X+f'*X;
    H = diag([ones(nvu, 1); zeros(nsu, 1)]);
    f = [zeros(nvu, 1); C*ones(nsu, 1)];
    A = [dp.AU [zeros(dp.hu, nsu); -eye(nsu)]; zeros(nsu, nvu) -eye(nsu)];
    b = [dp.BU-dp.AU*UC; zeros(nsu, 1)];
    X = qp(H, f, A, b);
    U = X(1:nvu)+UC;
    XIU = X(nvu+1:size(A,2));
    IU = 0.5*X'*H*X+f'*X;
  end
  nn = 0;
  alt = 0;
  EU = [];
  for ii = 1:length(dp.A)
    k = length(dp.A(ii).C);
    eu = dot(P(nn+1:nn+k), U(nn+1:nn+k));
    EU = [EU; eu];
    if (alt == 0 | eu > best_eu)
      best_eu = eu;
      alt = ii;
    end
    nn = nn + k;
  end